# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'taxonome/gui/ui/map_names_wizard.ui'
#
# Created: Tue May 29 11:46:18 2012
#      by: PyQt4 UI code generator 4.8.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_MapNamesWizard(object):
    def setupUi(self, MapNamesWizard):
        MapNamesWizard.setObjectName(_fromUtf8("MapNamesWizard"))
        MapNamesWizard.resize(555, 426)
        MapNamesWizard.setAutoFillBackground(False)
        MapNamesWizard.setSizeGripEnabled(True)
        self.datasets_page = QtGui.QWizardPage()
        self.datasets_page.setObjectName(_fromUtf8("datasets_page"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.datasets_page)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.groupBox_2 = QtGui.QGroupBox(self.datasets_page)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.verticalLayout_5 = QtGui.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.horizontalLayout_5 = QtGui.QHBoxLayout()
        self.horizontalLayout_5.setObjectName(_fromUtf8("horizontalLayout_5"))
        self.from_dataset = QtGui.QRadioButton(self.groupBox_2)
        self.from_dataset.setObjectName(_fromUtf8("from_dataset"))
        self.horizontalLayout_5.addWidget(self.from_dataset)
        self.taxa_dataset = QtGui.QComboBox(self.groupBox_2)
        self.taxa_dataset.setEnabled(False)
        self.taxa_dataset.setObjectName(_fromUtf8("taxa_dataset"))
        self.horizontalLayout_5.addWidget(self.taxa_dataset)
        self.verticalLayout_5.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        self.from_csv = QtGui.QRadioButton(self.groupBox_2)
        self.from_csv.setChecked(True)
        self.from_csv.setObjectName(_fromUtf8("from_csv"))
        self.horizontalLayout_6.addWidget(self.from_csv)
        self.from_csv_file = QtGui.QLineEdit(self.groupBox_2)
        self.from_csv_file.setEnabled(True)
        self.from_csv_file.setReadOnly(True)
        self.from_csv_file.setObjectName(_fromUtf8("from_csv_file"))
        self.horizontalLayout_6.addWidget(self.from_csv_file)
        self.from_csv_browse = QtGui.QPushButton(self.groupBox_2)
        self.from_csv_browse.setEnabled(True)
        self.from_csv_browse.setObjectName(_fromUtf8("from_csv_browse"))
        self.horizontalLayout_6.addWidget(self.from_csv_browse)
        self.verticalLayout_5.addLayout(self.horizontalLayout_6)
        self.label_4 = QtGui.QLabel(self.groupBox_2)
        self.label_4.setWordWrap(True)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.verticalLayout_5.addWidget(self.label_4)
        self.verticalLayout_4.addWidget(self.groupBox_2)
        self.groupBox_5 = QtGui.QGroupBox(self.datasets_page)
        self.groupBox_5.setObjectName(_fromUtf8("groupBox_5"))
        self.verticalLayout_12 = QtGui.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_12.setObjectName(_fromUtf8("verticalLayout_12"))
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.to_dataset = QtGui.QRadioButton(self.groupBox_5)
        self.to_dataset.setChecked(True)
        self.to_dataset.setObjectName(_fromUtf8("to_dataset"))
        self.horizontalLayout_4.addWidget(self.to_dataset)
        self.target_dataset = QtGui.QComboBox(self.groupBox_5)
        self.target_dataset.setObjectName(_fromUtf8("target_dataset"))
        self.horizontalLayout_4.addWidget(self.target_dataset)
        self.verticalLayout_12.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_10 = QtGui.QHBoxLayout()
        self.horizontalLayout_10.setObjectName(_fromUtf8("horizontalLayout_10"))
        self.to_tnrs = QtGui.QRadioButton(self.groupBox_5)
        self.to_tnrs.setObjectName(_fromUtf8("to_tnrs"))
        self.horizontalLayout_10.addWidget(self.to_tnrs)
        self.verticalLayout_12.addLayout(self.horizontalLayout_10)
        self.label_2 = QtGui.QLabel(self.groupBox_5)
        self.label_2.setWordWrap(True)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.verticalLayout_12.addWidget(self.label_2)
        self.verticalLayout_4.addWidget(self.groupBox_5)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem)
        MapNamesWizard.addPage(self.datasets_page)
        self.csv_import_page = QtGui.QWizardPage()
        self.csv_import_page.setSubTitle(_fromUtf8(""))
        self.csv_import_page.setObjectName(_fromUtf8("csv_import_page"))
        self.verticalLayout = QtGui.QVBoxLayout(self.csv_import_page)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.label_3 = QtGui.QLabel(self.csv_import_page)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.verticalLayout.addWidget(self.label_3)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label = QtGui.QLabel(self.csv_import_page)
        self.label.setTextFormat(QtCore.Qt.AutoText)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout.addWidget(self.label)
        self.namefield = QtGui.QComboBox(self.csv_import_page)
        self.namefield.setObjectName(_fromUtf8("namefield"))
        self.horizontalLayout.addWidget(self.namefield)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.groupBox = QtGui.QGroupBox(self.csv_import_page)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.noauth = QtGui.QRadioButton(self.groupBox)
        self.noauth.setObjectName(_fromUtf8("noauth"))
        self.verticalLayout_3.addWidget(self.noauth)
        self.auth_with_name = QtGui.QRadioButton(self.groupBox)
        self.auth_with_name.setObjectName(_fromUtf8("auth_with_name"))
        self.verticalLayout_3.addWidget(self.auth_with_name)
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.auth_separate = QtGui.QRadioButton(self.groupBox)
        self.auth_separate.setChecked(True)
        self.auth_separate.setObjectName(_fromUtf8("auth_separate"))
        self.horizontalLayout_3.addWidget(self.auth_separate)
        self.authfield = QtGui.QComboBox(self.groupBox)
        self.authfield.setEnabled(True)
        self.authfield.setObjectName(_fromUtf8("authfield"))
        self.horizontalLayout_3.addWidget(self.authfield)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.verticalLayout.addWidget(self.groupBox)
        self.preview_label = QtGui.QLabel(self.csv_import_page)
        self.preview_label.setObjectName(_fromUtf8("preview_label"))
        self.verticalLayout.addWidget(self.preview_label)
        self.preview = QtGui.QTableWidget(self.csv_import_page)
        self.preview.setObjectName(_fromUtf8("preview"))
        self.preview.setColumnCount(0)
        self.preview.setRowCount(0)
        self.verticalLayout.addWidget(self.preview)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.label_6 = QtGui.QLabel(self.csv_import_page)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.horizontalLayout_2.addWidget(self.label_6)
        self.csv_encoding = QtGui.QComboBox(self.csv_import_page)
        self.csv_encoding.setObjectName(_fromUtf8("csv_encoding"))
        self.horizontalLayout_2.addWidget(self.csv_encoding)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        MapNamesWizard.addPage(self.csv_import_page)
        self.matching_process_page = QtGui.QWizardPage()
        self.matching_process_page.setObjectName(_fromUtf8("matching_process_page"))
        self.verticalLayout_8 = QtGui.QVBoxLayout(self.matching_process_page)
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        self.groupBox_4 = QtGui.QGroupBox(self.matching_process_page)
        self.groupBox_4.setObjectName(_fromUtf8("groupBox_4"))
        self.verticalLayout_7 = QtGui.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_7.setObjectName(_fromUtf8("verticalLayout_7"))
        self.upgrade_none = QtGui.QRadioButton(self.groupBox_4)
        self.upgrade_none.setObjectName(_fromUtf8("upgrade_none"))
        self.verticalLayout_7.addWidget(self.upgrade_none)
        self.upgrade_nominal = QtGui.QRadioButton(self.groupBox_4)
        self.upgrade_nominal.setChecked(True)
        self.upgrade_nominal.setObjectName(_fromUtf8("upgrade_nominal"))
        self.verticalLayout_7.addWidget(self.upgrade_nominal)
        self.upgrade_all = QtGui.QRadioButton(self.groupBox_4)
        self.upgrade_all.setChecked(False)
        self.upgrade_all.setObjectName(_fromUtf8("upgrade_all"))
        self.verticalLayout_7.addWidget(self.upgrade_all)
        self.verticalLayout_8.addWidget(self.groupBox_4)
        self.groupBox_3 = QtGui.QGroupBox(self.matching_process_page)
        self.groupBox_3.setObjectName(_fromUtf8("groupBox_3"))
        self.verticalLayout_6 = QtGui.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_6.setObjectName(_fromUtf8("verticalLayout_6"))
        self.preferaccepted_all = QtGui.QRadioButton(self.groupBox_3)
        self.preferaccepted_all.setObjectName(_fromUtf8("preferaccepted_all"))
        self.verticalLayout_6.addWidget(self.preferaccepted_all)
        self.preferaccepted_noauth = QtGui.QRadioButton(self.groupBox_3)
        self.preferaccepted_noauth.setChecked(True)
        self.preferaccepted_noauth.setObjectName(_fromUtf8("preferaccepted_noauth"))
        self.verticalLayout_6.addWidget(self.preferaccepted_noauth)
        self.preferaccepted_none = QtGui.QRadioButton(self.groupBox_3)
        self.preferaccepted_none.setObjectName(_fromUtf8("preferaccepted_none"))
        self.verticalLayout_6.addWidget(self.preferaccepted_none)
        self.verticalLayout_8.addWidget(self.groupBox_3)
        self.auth_lax = QtGui.QCheckBox(self.matching_process_page)
        self.auth_lax.setChecked(True)
        self.auth_lax.setObjectName(_fromUtf8("auth_lax"))
        self.verticalLayout_8.addWidget(self.auth_lax)
        self.user_choice = QtGui.QCheckBox(self.matching_process_page)
        self.user_choice.setObjectName(_fromUtf8("user_choice"))
        self.verticalLayout_8.addWidget(self.user_choice)
        MapNamesWizard.addPage(self.matching_process_page)
        self.output_page = QtGui.QWizardPage()
        self.output_page.setObjectName(_fromUtf8("output_page"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.output_page)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.widget = QtGui.QWidget(self.output_page)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.verticalLayout_10 = QtGui.QVBoxLayout(self.widget)
        self.verticalLayout_10.setMargin(0)
        self.verticalLayout_10.setObjectName(_fromUtf8("verticalLayout_10"))
        self.taxadata = QtGui.QCheckBox(self.widget)
        self.taxadata.setChecked(True)
        self.taxadata.setObjectName(_fromUtf8("taxadata"))
        self.verticalLayout_10.addWidget(self.taxadata)
        self.horizontalWidget_5 = QtGui.QWidget(self.widget)
        self.horizontalWidget_5.setEnabled(True)
        self.horizontalWidget_5.setObjectName(_fromUtf8("horizontalWidget_5"))
        self.horizontalLayout_7 = QtGui.QHBoxLayout(self.horizontalWidget_5)
        self.horizontalLayout_7.setMargin(0)
        self.horizontalLayout_7.setObjectName(_fromUtf8("horizontalLayout_7"))
        self.taxadata_file = QtGui.QLineEdit(self.horizontalWidget_5)
        self.taxadata_file.setObjectName(_fromUtf8("taxadata_file"))
        self.horizontalLayout_7.addWidget(self.taxadata_file)
        self.taxadata_browse = QtGui.QPushButton(self.horizontalWidget_5)
        self.taxadata_browse.setObjectName(_fromUtf8("taxadata_browse"))
        self.horizontalLayout_7.addWidget(self.taxadata_browse)
        self.verticalLayout_10.addWidget(self.horizontalWidget_5)
        self.verticalLayout_2.addWidget(self.widget)
        self.widget_2 = QtGui.QWidget(self.output_page)
        self.widget_2.setObjectName(_fromUtf8("widget_2"))
        self.verticalLayout_11 = QtGui.QVBoxLayout(self.widget_2)
        self.verticalLayout_11.setMargin(0)
        self.verticalLayout_11.setObjectName(_fromUtf8("verticalLayout_11"))
        self.mappings = QtGui.QCheckBox(self.widget_2)
        self.mappings.setObjectName(_fromUtf8("mappings"))
        self.verticalLayout_11.addWidget(self.mappings)
        self.horizontalWidget_4 = QtGui.QWidget(self.widget_2)
        self.horizontalWidget_4.setEnabled(False)
        self.horizontalWidget_4.setObjectName(_fromUtf8("horizontalWidget_4"))
        self.horizontalLayout_9 = QtGui.QHBoxLayout(self.horizontalWidget_4)
        self.horizontalLayout_9.setMargin(0)
        self.horizontalLayout_9.setObjectName(_fromUtf8("horizontalLayout_9"))
        self.mappings_file = QtGui.QLineEdit(self.horizontalWidget_4)
        self.mappings_file.setObjectName(_fromUtf8("mappings_file"))
        self.horizontalLayout_9.addWidget(self.mappings_file)
        self.mappings_browse = QtGui.QPushButton(self.horizontalWidget_4)
        self.mappings_browse.setObjectName(_fromUtf8("mappings_browse"))
        self.horizontalLayout_9.addWidget(self.mappings_browse)
        self.verticalLayout_11.addWidget(self.horizontalWidget_4)
        self.verticalLayout_2.addWidget(self.widget_2)
        self.widget_3 = QtGui.QWidget(self.output_page)
        self.widget_3.setObjectName(_fromUtf8("widget_3"))
        self.verticalLayout_9 = QtGui.QVBoxLayout(self.widget_3)
        self.verticalLayout_9.setMargin(0)
        self.verticalLayout_9.setObjectName(_fromUtf8("verticalLayout_9"))
        self.log = QtGui.QCheckBox(self.widget_3)
        self.log.setObjectName(_fromUtf8("log"))
        self.verticalLayout_9.addWidget(self.log)
        self.horizontalWidget_3 = QtGui.QWidget(self.widget_3)
        self.horizontalWidget_3.setEnabled(False)
        self.horizontalWidget_3.setObjectName(_fromUtf8("horizontalWidget_3"))
        self.horizontalLayout_8 = QtGui.QHBoxLayout(self.horizontalWidget_3)
        self.horizontalLayout_8.setMargin(0)
        self.horizontalLayout_8.setObjectName(_fromUtf8("horizontalLayout_8"))
        self.log_file = QtGui.QLineEdit(self.horizontalWidget_3)
        self.log_file.setObjectName(_fromUtf8("log_file"))
        self.horizontalLayout_8.addWidget(self.log_file)
        self.log_browse = QtGui.QPushButton(self.horizontalWidget_3)
        self.log_browse.setObjectName(_fromUtf8("log_browse"))
        self.horizontalLayout_8.addWidget(self.log_browse)
        self.verticalLayout_9.addWidget(self.horizontalWidget_3)
        self.verticalLayout_2.addWidget(self.widget_3)
        self.matched_ds = QtGui.QCheckBox(self.output_page)
        self.matched_ds.setObjectName(_fromUtf8("matched_ds"))
        self.verticalLayout_2.addWidget(self.matched_ds)
        self.label_5 = QtGui.QLabel(self.output_page)
        self.label_5.setWordWrap(True)
        self.label_5.setObjectName(_fromUtf8("label_5"))
        self.verticalLayout_2.addWidget(self.label_5)
        MapNamesWizard.addPage(self.output_page)
        self.label.setBuddy(self.namefield)
        self.preview_label.setBuddy(self.preview)

        self.retranslateUi(MapNamesWizard)
        QtCore.QObject.connect(self.from_dataset, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.taxa_dataset.setEnabled)
        QtCore.QObject.connect(self.from_csv, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.from_csv_file.setEnabled)
        QtCore.QObject.connect(self.from_csv, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.from_csv_browse.setEnabled)
        QtCore.QObject.connect(self.auth_separate, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.authfield.setEnabled)
        QtCore.QObject.connect(self.taxadata, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.horizontalWidget_5.setEnabled)
        QtCore.QObject.connect(self.mappings, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.horizontalWidget_4.setEnabled)
        QtCore.QObject.connect(self.log, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.horizontalWidget_3.setEnabled)
        QtCore.QObject.connect(self.to_dataset, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.target_dataset.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(MapNamesWizard)

    def retranslateUi(self, MapNamesWizard):
        MapNamesWizard.setWindowTitle(QtGui.QApplication.translate("MapNamesWizard", "Match taxa by name", None, QtGui.QApplication.UnicodeUTF8))
        self.datasets_page.setTitle(QtGui.QApplication.translate("MapNamesWizard", "Matching names", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("MapNamesWizard", "Match taxa from:", None, QtGui.QApplication.UnicodeUTF8))
        self.from_dataset.setText(QtGui.QApplication.translate("MapNamesWizard", "Dataset", None, QtGui.QApplication.UnicodeUTF8))
        self.from_csv.setText(QtGui.QApplication.translate("MapNamesWizard", "CSV file", None, QtGui.QApplication.UnicodeUTF8))
        self.from_csv_browse.setText(QtGui.QApplication.translate("MapNamesWizard", "Browse...", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("MapNamesWizard", "Reading directly from a CSV file is recommended for very large datasets, which may not fit into the computer\'s memory, or for datasets of individual samples, specimens etc. where the same name may appear several times.", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_5.setTitle(QtGui.QApplication.translate("MapNamesWizard", "Match to names in:", None, QtGui.QApplication.UnicodeUTF8))
        self.to_dataset.setText(QtGui.QApplication.translate("MapNamesWizard", "Dataset", None, QtGui.QApplication.UnicodeUTF8))
        self.to_tnrs.setText(QtGui.QApplication.translate("MapNamesWizard", "Taxonomic Name Resolution Service (iPlant Collaborative)", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("MapNamesWizard", "Using the TNRS with data from a CSV file will run through the file twice, first to collect the names, then to match each line.", None, QtGui.QApplication.UnicodeUTF8))
        self.csv_import_page.setTitle(QtGui.QApplication.translate("MapNamesWizard", "CSV columns", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("MapNamesWizard", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'MS Shell Dlg 2\'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:8pt;\">Please select which columns contain the names (e.g. </span><span style=\" font-size:8pt; font-style:italic;\">Vigna radiata</span><span style=\" font-size:8pt;\">) and the names\' authorities (e.g. \'(L.) R. Wilczek\').</span></p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("MapNamesWizard", "Name column:", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("MapNamesWizard", "Authorities:", None, QtGui.QApplication.UnicodeUTF8))
        self.noauth.setText(QtGui.QApplication.translate("MapNamesWizard", "None", None, QtGui.QApplication.UnicodeUTF8))
        self.auth_with_name.setText(QtGui.QApplication.translate("MapNamesWizard", "In column with species", None, QtGui.QApplication.UnicodeUTF8))
        self.auth_separate.setText(QtGui.QApplication.translate("MapNamesWizard", "In separate column:", None, QtGui.QApplication.UnicodeUTF8))
        self.preview_label.setText(QtGui.QApplication.translate("MapNamesWizard", "Preview", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("MapNamesWizard", "Character encoding", None, QtGui.QApplication.UnicodeUTF8))
        self.matching_process_page.setTitle(QtGui.QApplication.translate("MapNamesWizard", "Name matching options", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_4.setTitle(QtGui.QApplication.translate("MapNamesWizard", "Subspecies and varieties", None, QtGui.QApplication.UnicodeUTF8))
        self.upgrade_none.setText(QtGui.QApplication.translate("MapNamesWizard", "Strict matches only", None, QtGui.QApplication.UnicodeUTF8))
        self.upgrade_nominal.setToolTip(QtGui.QApplication.translate("MapNamesWizard", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">e.g. <span style=\" font-style:italic;\">Zea mays </span>subsp. <span style=\" font-style:italic;\">mays</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">would match <span style=\" font-style:italic;\">Zea mays</span>.</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.upgrade_nominal.setText(QtGui.QApplication.translate("MapNamesWizard", "Match nominal subspecies to species", None, QtGui.QApplication.UnicodeUTF8))
        self.upgrade_all.setToolTip(QtGui.QApplication.translate("MapNamesWizard", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">e.g. <span style=\" font-style:italic;\">Zea mays </span>subsp. <span style=\" font-style:italic;\">parviglumis</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">would match <span style=\" font-style:italic;\">Zea mays</span>.</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.upgrade_all.setText(QtGui.QApplication.translate("MapNamesWizard", "Match any subspecies to species", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setTitle(QtGui.QApplication.translate("MapNamesWizard", "Automatically select accepted name", None, QtGui.QApplication.UnicodeUTF8))
        self.preferaccepted_all.setText(QtGui.QApplication.translate("MapNamesWizard", "Always", None, QtGui.QApplication.UnicodeUTF8))
        self.preferaccepted_noauth.setText(QtGui.QApplication.translate("MapNamesWizard", "When authority info is missing", None, QtGui.QApplication.UnicodeUTF8))
        self.preferaccepted_none.setText(QtGui.QApplication.translate("MapNamesWizard", "Never", None, QtGui.QApplication.UnicodeUTF8))
        self.auth_lax.setToolTip(QtGui.QApplication.translate("MapNamesWizard", "If this is selected, when there is only one match for the name, it will be used even if the authority doesn\'t match.", None, QtGui.QApplication.UnicodeUTF8))
        self.auth_lax.setText(QtGui.QApplication.translate("MapNamesWizard", "Allow mismatched authorities", None, QtGui.QApplication.UnicodeUTF8))
        self.user_choice.setText(QtGui.QApplication.translate("MapNamesWizard", "Manually choose between ambiguous matches", None, QtGui.QApplication.UnicodeUTF8))
        self.output_page.setTitle(QtGui.QApplication.translate("MapNamesWizard", "Output options", None, QtGui.QApplication.UnicodeUTF8))
        self.taxadata.setText(QtGui.QApplication.translate("MapNamesWizard", "Taxa data with new names", None, QtGui.QApplication.UnicodeUTF8))
        self.taxadata_browse.setText(QtGui.QApplication.translate("MapNamesWizard", "Browse...", None, QtGui.QApplication.UnicodeUTF8))
        self.mappings.setText(QtGui.QApplication.translate("MapNamesWizard", "Name mappings", None, QtGui.QApplication.UnicodeUTF8))
        self.mappings_browse.setText(QtGui.QApplication.translate("MapNamesWizard", "Browse...", None, QtGui.QApplication.UnicodeUTF8))
        self.log.setText(QtGui.QApplication.translate("MapNamesWizard", "Full log of matching process", None, QtGui.QApplication.UnicodeUTF8))
        self.log_browse.setText(QtGui.QApplication.translate("MapNamesWizard", "Browse...", None, QtGui.QApplication.UnicodeUTF8))
        self.matched_ds.setText(QtGui.QApplication.translate("MapNamesWizard", "Create dataset of source taxa with new names", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("MapNamesWizard", "This is not recommended when matching big datasets read directly from file.", None, QtGui.QApplication.UnicodeUTF8))




"""GUI Dialogs for mapping one set of names to another.
"""
import csv
import os.path
from functools import partial

from .qt import QtGui, QtCore

from taxonome.taxa import match_taxa
from taxonome.taxa import file_csv
from taxonome.taxa.collection import run_match_taxa
from taxonome import tracker
from .ui.map_names_wizard import Ui_MapNamesWizard
from .csvdialogs import CsvImportDialog, supported_encodings
from .iothread import Worker, makeloader

class MapNamesWizard(QtGui.QWizard, CsvImportDialog):
    def __init__(self, app=None):
        QtGui.QWizard.__init__(self, parent=app)
        self.ui = Ui_MapNamesWizard()
        self.ui.setupUi(self)
        
        # Hook up to main application
        self.app = app
        if app:
            self.lastdir = app.lastdir
            self.ui.taxa_dataset.setModel(app.datasets_model)
            self.ui.target_dataset.setModel(app.datasets_model)
        else:
            self.lastdir = os.path.expanduser("~")
        
        # When should we skip certain pages?
        self.ui.datasets_page.nextId = self.select_dataset_next_page
        self.ui.csv_import_page.nextId = self.csv_opts_next_page
        
        # Encoding switcher for CSV file
        # The signal from QComboBox is overloaded - we want the str form
        self.ui.csv_encoding.addItems(supported_encodings)
        QtCore.QObject.connect(self.ui.csv_encoding,
            QtCore.SIGNAL("currentIndexChanged(QString)"), self.encoding_changed)
        
        # Browse buttons for input & output files
        self.ui.from_csv_browse.clicked.connect(self.select_csv_in)
        self.ui.taxadata_browse.clicked.connect(self._select_save_location(self.ui.taxadata_file))
        self.ui.mappings_browse.clicked.connect(self._select_save_location(self.ui.mappings_file))
        self.ui.log_browse.clicked.connect(self._select_save_location(self.ui.log_file))
    
    def select_csv_in(self):
        filename = QtGui.QFileDialog.getOpenFileName(self, "Select big CSV file",
                                    self.lastdir, "CSV (*.csv);;All files (*.*)")
        if not filename:
            return
        self.ui.from_csv_file.setText(filename)
        self.preview_file()
    
    @property
    def filename(self):
        """Return the CSV input filename, for the methods relating to that."""
        return self.ui.from_csv_file.text()
    
    def select_dataset_next_page(self):
        """Skip the CSV import page if we're using taxa from a local dataset."""
        if self.ui.from_csv.isChecked():
            return 1
        else:
            return self.csv_opts_next_page()
    
    def csv_opts_next_page(self):
        """Skip the matching options if we're using TNRS."""
        return 3 if self.ui.to_tnrs.isChecked() else 2

    def _select_save_location(self, destination):
        """Make a function to get an output filename and put it in a text box."""
        def inner():
            filename = QtGui.QFileDialog.getSaveFileName(self, "Save file to:",
                                self.lastdir, "CSV (*.csv);;All files (*.*)")
            if filename:
                if self.app:
                    self.app.lastdir = os.path.dirname(filename)
                
                if not os.path.splitext(filename)[1]:
                    filename += '.csv'
                destination.setText(filename)
        return inner
    
    # Properties to see which radio button is selected
    @property
    def upgrade_option(self):
        if self.ui.upgrade_all.isChecked():
            return 'all'
        elif self.ui.upgrade_none.isChecked():
            return 'none'
        else:
            return 'nominal'
    
    @property
    def preferaccepted_option(self):
        if self.ui.preferaccepted_all.isChecked():
            prefer_accepted = 'all'
        elif self.ui.preferaccepted_none.isChecked():
            prefer_accepted = 'none'
        else:
            prefer_accepted = 'noauth'

def wizard_map_names(app):
    """Match a big dataset (a CSV file, can be too big to hold in memory) against
    one of the loaded datasets.
    
    Presents the user with a set of options for controlling input, matching and
    output.
    """
    wizard = MapNamesWizard(app)
    
    # Run the wizard
    if not wizard.exec_():
        return False
    
    # Get the options ---------------------------------------------------------
    seln = wizard.ui.target_dataset.currentIndex()       # 1. From & to
    target = app.datasets_model.item(seln)
    if wizard.ui.from_csv.isChecked():
        csv_filename = wizard.ui.from_csv_file.text()
        from_csv = True
    else:
        seln = wizard.ui.taxa_dataset.currentIndex()
        dataset_item = app.datasets_model.item(seln)
        taxa_dataset = dataset_item.ds
        from_csv = False
    to_tnrs = wizard.ui.to_tnrs.isChecked()
    
    namefield = wizard.ui.namefield.currentText()         # 2. CSV
    authfield = wizard.get_auth_field()
    csv_encoding = wizard.ui.csv_encoding.currentText()
    
    upgrade = wizard.upgrade_option                       # 3. Matching options
    prefer_accepted = wizard.preferaccepted_option
    strict_authority = not wizard.ui.auth_lax.isChecked()
    user_choice = wizard.ui.user_choice.isChecked()
    
    taxadata_file, mappings_file, log_file = None, None, None  # 4. Output
    if wizard.ui.taxadata.isChecked():
        taxadata_file = wizard.ui.taxadata_file.text()
    if wizard.ui.mappings.isChecked():
        mappings_file = wizard.ui.mappings_file.text()
    if wizard.ui.log.isChecked():
        log_file = wizard.ui.log_file.text()
    matched_ds = wizard.ui.matched_ds.isChecked()
    
    # Prepare for matching ----------------------------------------------------
    nameselector = app.gui_select_name if user_choice else app.silent_select_name
    
    trackers = []
    files = []
    if mappings_file:
        f = open(mappings_file, "w", encoding='utf-8', newline='')
        files.append(f)
        trackers.append(tracker.CSVListMatches(f))
    if log_file:
        f = open(log_file, "w", encoding='utf-8', newline='')
        files.append(f)
        trackers.append(tracker.CSVTracker(f))
    if taxadata_file:
        if from_csv:
            with open(csv_filename, encoding=csv_encoding, errors='replace', newline='') as f:
                fields = next(csv.reader(f))
            fields.remove(namefield)
            if isinstance(authfield, str):
                fields.remove(authfield)
        else:
            fields = set()
            for t in taxa_dataset: fields.update(t.info)
            fields = list(fields)
        f = open(taxadata_file, "w", encoding='utf-8', newline='')
        files.append(f)
        trackers.append(tracker.CSVTaxaTracker(f, fields))
    
    def _run(progress=None):
        """Called in a separate thread to run the matching."""
        if from_csv:
            csvfile = open(csv_filename, encoding=csv_encoding, errors='replace')
            files.append(csvfile)
            data_in = file_csv.iter_taxa(csvfile, namefield=namefield, authfield=authfield, tracker=trackers, progress=progress)
        else:
            if progress:
                trackers.append(tracker.Counter(progress))
            data_in = taxa_dataset
        
        if to_tnrs:
            # Match to TNRS
            if from_csv:
                csvfile2 = open(csv_filename, encoding=csv_encoding, errors='replace')
                files.append(csvfile2)
                data_in2 = file_csv.iter_taxa(csvfile2, namefield=namefield, authfield=authfield)
            else:
                data_in2 = None
            
            from taxonome.services import tnrs
            # Do we want the output as a dataset?
            matchfunc = tnrs.match_taxa if matched_ds else tnrs.run_match_taxa
            # Run the matching
            res = matchfunc(data_in, data_in2, tracker=trackers)
        
        else:
            # Match to dataset
            
            # Do we want the output as a dataset?
            matchfunc = match_taxa if matched_ds else run_match_taxa
            # Run the matching
            res = matchfunc(data_in, target.ds, upgrade_subsp=upgrade,
                strict_authority=strict_authority, nameselector=nameselector, tracker=trackers)
        
        for f in files: f.close()
        return res
    
    # Run the matching in another thread --------------------------------------
    steps = os.stat(csv_filename).st_size if from_csv else len(taxa_dataset)
    if matched_ds:
        fromname = os.path.basename(csv_filename) if from_csv else dataset_item.name
        ds_name = "{} mapped to {}".format(fromname, target.name)
        makeloader(app, _run, ds_name, steps)
    else:
        thread = Worker(_run, app)
        thread.error_raised.connect(app.show_error)
        thread.withprogress(app, steps)
        thread.start()



from ..digraph import DiGraph
import unittest

class SimpleTestCase(unittest.TestCase):
    def setUp(self):
        self.g = DiGraph()
        
    def test_add_node(self):
        self.g.nodes.add(1)
        self.g.nodes.add("spam")
        self.assertIn(1, self.g.nodes)
        self.assertNotIn(2, self.g.nodes)
        self.assertIn("spam", self.g.nodes)
        self.assertNotIn("eggs", self.g.nodes)
        
    def test_add_edge(self):
        self.g.nodes.add(1)
        self.g.nodes.add(2)
        self.g.edges.add(1, 2)
        self.g.edges.add(3, 4)
        self.assertIn(3, self.g.nodes)
        self.assertIn(4, self.g.nodes)
        self.g.edges.add(1, 3)
        
        self.assertIn((1,3), self.g.edges)
        self.assertIn((3, 4), self.g.edges)
        self.assertNotIn((3, 1), self.g.edges)
        self.assertNotIn((1, 4), self.g.edges)


class BuildTestCase(unittest.TestCase):
    def test_build_nodes(self):
        g = DiGraph.build([], range(6))
        self.assertEqual(len(g.nodes), 6)
        self.assertEqual(len(g.edges), 0)
        
    def test_build_edges(self):
        edges = zip(range(5), range(1, 6))
        g = DiGraph.build(edges)
        self.assertEqual(len(g.nodes), 6)
        self.assertEqual(len(g.edges), 5)
        self.assertIn((1,2), g.edges)
        self.assertNotIn((1,3), g.edges)
        self.assertNotIn((2,1), g.edges)
        
    def test_build_mixed(self):
        edges = zip(range(5), range(1, 6))
        g = DiGraph.build(edges, range(3, 8))
        self.assertEqual(len(g.nodes), 8)
        self.assertEqual(len(g.edges), 5)

class LinksTestCase(unittest.TestCase):
    def setUp(self):
        edges = zip(range(5), range(1, 6))
        self.g = DiGraph.build(edges)
        self.g.edges.add(2, 5)
    
    def test_incoming(self):
        self.assertEqual(len(self.g.incoming(0)), 0)
        self.assertEqual(len(self.g.incoming(3)), 1)
        in3 = set(self.g.incoming(3))
        self.assertEqual(in3, {2})
        
    def test_outgoing(self):
        self.assertEqual(len(self.g.outgoing(5)), 0)
        self.assertEqual(len(self.g.outgoing(3)), 1)
        out3 = set(self.g.outgoing(3))
        self.assertEqual(out3, {4})
        
    def test_successors(self):
        succ1 = list(self.g.successors(1))
        self.assertEqual(len(succ1), 4)
        self.assertEqual(set(succ1), {2,3,4,5})
        
        self.assertTrue(self.g.is_successor(2, 5))
        self.assertTrue(self.g.is_successor(3, 5))
        self.assertFalse(self.g.is_successor(5, 3))
        self.assertFalse(self.g.is_successor(5, 7))   # There is no 7
        self.assertTrue(self.g.is_successor(1, 2))
        self.assertFalse(self.g.is_successor(2, 2))
        
    def test_infinite_loop(self):
        "Make a cycle, and check that it doesn't hang. May need to be killed."
        self.g.edges.add(5, 3)
        succ2 = set(self.g.successors(2))
        self.assertEqual(succ2, {3,4,5})
        for x in succ2:
            self.assertTrue(self.g.is_successor(2, x))
            self.assertFalse(self.g.is_successor(x, 2))
            
            self.assertTrue(self.g.is_successor(3, x))
            self.assertTrue(self.g.is_successor(x, 3))
            
if __name__ == "__main__":
    unittest.main()



# coding: utf-8

from .qt import QtCore, QtGui

from .ui.combine import Ui_CombineDialog
from taxonome.taxa import combine_datasets
from taxonome.taxa.combine import add_info_flat, add_info_nested

def _get_items(model):
    """Extract items from a single column QStandardItemModel"""
    return (model.item(r) for r in range(model.rowCount()))
    
_distrib_ds_all = "(Combine any distribution data)"

class CombineDialog(QtGui.QDialog):
    def __init__(self, parent):
        QtGui.QDialog.__init__(self)
        self.ui = Ui_CombineDialog()
        self.ui.setupUi(self)
        self.parent = parent
        
        ds_model = parent.datasets_model
        
        # Set up the dataset lists
        self.ui.available_ds_list.setModel(ds_model)
        self.target_ds = QtGui.QStandardItemModel()
        self.bg_ds = QtGui.QStandardItemModel()
        self.ui.target_ds_list.setModel(self.target_ds)
        self.ui.bg_ds_list.setModel(self.bg_ds)
        
        # Set up combobox for distribution
        self.ui.distrib_ds.addItem(_distrib_ds_all)
        self.ui.distrib_ds.addItems([dsi.name for dsi in _get_items(ds_model)])
    
    def add_target_ds(self):
        dsi = self.get_selected_available_ds()
        self.target_ds.appendRow(dsi.copy())
        
    def add_bg_ds(self):
        dsi = self.get_selected_available_ds()
        self.bg_ds.appendRow(dsi.copy())
    
    def remove_target_ds(self):
        seln = self.ui.target_ds_list.selectedIndexes()
        if seln:
            self.target_ds.removeRow(seln[0].row())
    
    def remove_bg_ds(self):
        seln = self.ui.bg_ds_list.selectedIndexes()
        if seln:
            self.bg_ds.removeRow(seln[0].row())
    
    def get_selected_available_ds(self):
        """Get the selected DatasetItem from the list of available datasets"""
        seln = self.ui.available_ds_list.selectedIndexes()
        if not seln:
            return None
        return self.parent.datasets_model.itemFromIndex(seln[0])

def gui_combine_datasets(app):
    dialog = CombineDialog(app)
    if not dialog.exec_():
        return
    
    newname = dialog.ui.newname.text()
    target_ds = [(dsi.name, dsi.ds) for dsi in _get_items(dialog.target_ds)]
    bg_ds = [(dsi.name, dsi.ds) for dsi in _get_items(dialog.bg_ds)]
    distrib_ds = dialog.ui.distrib_ds.currentText()
    if distrib_ds == _distrib_ds_all:
        distrib_ds = "all"
    
    add_info = add_info_nested if dialog.ui.nest_info.isChecked() \
          else add_info_flat
    
    new_ds = combine_datasets(target_ds, bg_ds, distrib_ds, add_info=add_info)
    
    app.add_dataset(newname, new_ds)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'taxonome/gui/ui/csvimport.ui'
#
# Created: Tue May 29 11:46:18 2012
#      by: PyQt4 UI code generator 4.8.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_CsvImportDialog(object):
    def setupUi(self, CsvImportDialog):
        CsvImportDialog.setObjectName(_fromUtf8("CsvImportDialog"))
        CsvImportDialog.resize(435, 528)
        self.verticalLayout = QtGui.QVBoxLayout(CsvImportDialog)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label_3 = QtGui.QLabel(CsvImportDialog)
        self.label_3.setTextFormat(QtCore.Qt.AutoText)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.horizontalLayout.addWidget(self.label_3)
        self.ds_name = QtGui.QLineEdit(CsvImportDialog)
        self.ds_name.setObjectName(_fromUtf8("ds_name"))
        self.horizontalLayout.addWidget(self.ds_name)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.label_4 = QtGui.QLabel(CsvImportDialog)
        self.label_4.setWordWrap(True)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.verticalLayout.addWidget(self.label_4)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.label = QtGui.QLabel(CsvImportDialog)
        self.label.setTextFormat(QtCore.Qt.AutoText)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout_2.addWidget(self.label)
        self.namefield = QtGui.QComboBox(CsvImportDialog)
        self.namefield.setObjectName(_fromUtf8("namefield"))
        self.horizontalLayout_2.addWidget(self.namefield)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.groupBox = QtGui.QGroupBox(CsvImportDialog)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.noauth = QtGui.QRadioButton(self.groupBox)
        self.noauth.setObjectName(_fromUtf8("noauth"))
        self.verticalLayout_2.addWidget(self.noauth)
        self.auth_with_name = QtGui.QRadioButton(self.groupBox)
        self.auth_with_name.setObjectName(_fromUtf8("auth_with_name"))
        self.verticalLayout_2.addWidget(self.auth_with_name)
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.auth_separate = QtGui.QRadioButton(self.groupBox)
        self.auth_separate.setChecked(True)
        self.auth_separate.setObjectName(_fromUtf8("auth_separate"))
        self.horizontalLayout_3.addWidget(self.auth_separate)
        self.authfield = QtGui.QComboBox(self.groupBox)
        self.authfield.setEnabled(True)
        self.authfield.setObjectName(_fromUtf8("authfield"))
        self.horizontalLayout_3.addWidget(self.authfield)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.verticalLayout.addWidget(self.groupBox)
        self.label_2 = QtGui.QLabel(CsvImportDialog)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.verticalLayout.addWidget(self.label_2)
        self.preview = QtGui.QTableWidget(CsvImportDialog)
        self.preview.setObjectName(_fromUtf8("preview"))
        self.preview.setColumnCount(0)
        self.preview.setRowCount(0)
        self.verticalLayout.addWidget(self.preview)
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.label_5 = QtGui.QLabel(CsvImportDialog)
        self.label_5.setObjectName(_fromUtf8("label_5"))
        self.horizontalLayout_4.addWidget(self.label_5)
        self.csv_encoding = QtGui.QComboBox(CsvImportDialog)
        self.csv_encoding.setObjectName(_fromUtf8("csv_encoding"))
        self.horizontalLayout_4.addWidget(self.csv_encoding)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.buttonBox = QtGui.QDialogButtonBox(CsvImportDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName(_fromUtf8("buttonBox"))
        self.verticalLayout.addWidget(self.buttonBox)
        self.label_3.setBuddy(self.ds_name)
        self.label.setBuddy(self.namefield)
        self.label_2.setBuddy(self.preview)

        self.retranslateUi(CsvImportDialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("accepted()")), CsvImportDialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("rejected()")), CsvImportDialog.reject)
        QtCore.QObject.connect(self.auth_separate, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.authfield.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(CsvImportDialog)

    def retranslateUi(self, CsvImportDialog):
        CsvImportDialog.setWindowTitle(QtGui.QApplication.translate("CsvImportDialog", "Import CSV", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("CsvImportDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'MS Shell Dlg 2\'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Ubuntu\'; font-size:8pt;\">Dataset name:</span></p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("CsvImportDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'MS Shell Dlg 2\'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:8pt;\">Please select which columns contain the name (e.g. </span><span style=\" font-size:8pt; font-style:italic;\">Vigna radiata</span><span style=\" font-size:8pt;\">) and the name authorities (e.g. \'(L.) R. Wilczek\').</span></p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("CsvImportDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'MS Shell Dlg 2\'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Ubuntu\'; font-size:8pt;\">Name column:</span></p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("CsvImportDialog", "Authorities:", None, QtGui.QApplication.UnicodeUTF8))
        self.noauth.setText(QtGui.QApplication.translate("CsvImportDialog", "None", None, QtGui.QApplication.UnicodeUTF8))
        self.auth_with_name.setText(QtGui.QApplication.translate("CsvImportDialog", "In column with species", None, QtGui.QApplication.UnicodeUTF8))
        self.auth_separate.setText(QtGui.QApplication.translate("CsvImportDialog", "In separate column:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("CsvImportDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Preview</span></p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("CsvImportDialog", "Character encoding", None, QtGui.QApplication.UnicodeUTF8))




# -*- coding: utf8 -*-
from taxonome.taxa.base import Name
import csv
from collections import defaultdict

def _no_op(*args, **kwargs): pass

class NoopTracker:
    """This tracker doesn't do anything."""
    name_event = name_transform = start_taxon = reset = unreadable_name = _no_op

noop_tracker = NoopTracker()

class MultiTracker:
    """Holds a list of trackers, and calls the relevant methods of each of them
    when its methods are called."""
    def __init__(self, *args):
        self._trackers = list(args)
    
    def __getattr__(self, name):
        methods = [getattr(t, name) for t in self._trackers]
        def dispatch(*args, **kwargs):
            for m in methods:
                m(*args, **kwargs)
        return dispatch

def prepare_tracker(tracker):
    if isinstance(tracker, (list, tuple)):
        return MultiTracker(*tracker)
    if tracker is None:
        return noop_tracker
    return tracker

def add_tracker(current, new):
    if isinstance(current, MultiTracker):
        current._trackers.append(new)
        return current
    if current is noop_tracker:
        return new
    
    return MultiTracker(current, new)

def coroutine(func):
    """Decorator to prime coroutines"""
    def start(*args,**kwargs):
        cr = func(*args,**kwargs)
        next(cr)
        return cr
    return start

RESET = object()
@coroutine
def csvchainwriter(writer):
    """Coroutine to write lines to a csv file, incrementing a chain number until
    it's sent a reset signal."""
    chain = 0
    while True:
        line = (yield)
        if line is RESET:
            chain = 0
        else:
            writer.writerow((chain,) + tuple(line))
            chain += 1

def _flatten_name(name):
    if isinstance(name, Name):
        return name.plain, str(name.authority)
    return name, None

class CSVTracker(NoopTracker):
    """This writes events to a CSV file."""
    def __init__(self, fileobj, header=True):
        writer = csv.writer(fileobj)
        if header:
            writer.writerow(["sequence", "From name", "From authority", "To name", "To authority", "Event", "Details"])
        self.push = csvchainwriter(writer).send
        
    def name_event(self, name, event, **kwargs):
        self.push(_flatten_name(name) + (None, None, event, kwargs or None))
    
    def name_transform(self, name, newname, event, **kwargs):
        self.push(_flatten_name(name) + _flatten_name(newname) + (event, kwargs or None))
    
    def reset(self):
        self.push(RESET)
    
    def unreadable_name(self, rawname, error):
        self.push((rawname, None, None, None, 'error reading name', error))

class CSVListMatches(NoopTracker):
    """Writes the original name and final match to a CSV file."""
    fromname = None
    toname = None
    
    def __init__(self, fileobj, header=True):
        self.writer = csv.writer(fileobj)
        if header:
            self.writer.writerow(["Original Name","Original Authority","Name", "Authority", "Score"])
    
    def start_taxon(self, tax):
        self.fromname = tax.name
        self.fuzzyscore = 1
        
    def name_event(self, name, event):
        self.toname = None
    
    def name_transform(self, name, newname, event, **kwargs):
        self.toname = newname
        if event == 'fuzzy match':
            score = kwargs.get('score', 1)
            self.fuzzyscore = min(score, self.fuzzyscore)
    
    def reset(self):
        if self.fromname:
            score = (self.fuzzyscore,) if self.toname else (None,)
            self.writer.writerow(_flatten_name(self.fromname) + _flatten_name(self.toname) + score)
        self.fromname = None
        self.toname = None
    
    def unreadable_name(self, rawname, error):
        self.fromname = rawname

class Counter(NoopTracker):
    """Count the number of names seen so far. Callback with the number after
    every n (default 10).
    """
    def __init__(self, callback, every=10):
        self.callback = callback
        self.every = every
        self.n = 0
    
    def start_taxon(self, tax):
        self.n += 1
        if self.n % self.every == 0:
            self.callback(self.n)

class EventCounter(NoopTracker):
    def __init__(self):
        self.started = 0
        self.read_failed = 0
        self.events = defaultdict(int)
    
    def start_taxon(self, tax):
        self.started += 1
    
    def name_event(self, name, event):
        self.events[event] += 1
    
    def name_transform(self, name, newname, event, **kwargs):
        self.events[event] += 1
    
    def unreadable_name(self, rawname, error):
        self.read_failed += 1

class CSVTaxaTracker(NoopTracker):
    """Produces a list of the existing taxa data with the new names."""
    taxon = None
    newname = None
    
    def __init__(self, fileobj, fieldnames, header=True, include_unmatched=False):
        fieldnames = ["Name", "Authority", "Original name", "Original authority"]+fieldnames
        self.writer = csv.DictWriter(fileobj, fieldnames, extrasaction='ignore')
        if header:
            self.writer.writeheader()
        
        self.include_unmatched = include_unmatched
    
    def start_taxon(self, tax):
        self.taxon = tax
        self.fuzzyscore = 1
    
    def name_transform(self, name, newname, event, **kwargs):
        self.newname = newname
    
    def reset(self):
        if self.taxon and (self.newname or self.include_unmatched):
            d = dict(self.taxon.info)
            d['Original name'], d['Original authority'] = _flatten_name(self.taxon.name)
            d['Name'], d['Authority']= _flatten_name(self.newname)
            self.writer.writerow(d)
        
        self.taxon = None
        self.newname = None
    
    def unreadable_name(self, rawname, error):
        if self.include_unmatched:
            self.writer.writerow({'Original name':rawname})

class CacheTracker(NoopTracker):
    """For internal use: cache matching results so they can be reused."""
    def __init__(self):
        self.cache = {}
    
    def start_taxon(self, tax):
        self.fromname = tax.name
        self.toname = None
        self.events = []
    
    def reset(self):
        if self.fromname not in self.cache:
            self.cache[self.fromname] = self.events, self.toname
    
    def retrieve(self, fromname, tracker):
        """Retrieve a previously-seen name, and replay its events to the
        tracker.
        
        Raises KeyError if the name hasn't been seen before.
        """
        events, toname = self.cache[fromname]
        for meth, *args, kwargs in events:
            getattr(tracker, meth)(*args, **kwargs)
    
    def name_event(self, name, event, **kwargs):
        if event == 'no match':
            self.toname = None
        self.events.append(('name_event', name, event, kwargs))
    
    def name_transform(self, name, newname, event, **kwargs):
        self.toname = newname
        self.events.append(('name_transform', name, newname, event, kwargs))
        
        
        
        
        
        
        
        
        
        
        
        
        
        



import csv
from io import StringIO
import unittest

from taxonome import Taxon, TaxonSet
from taxonome.taxa import match_taxa
from taxonome import tracker

class TrackerTest(unittest.TestCase):
    def setUp(self):
        self.ts = TaxonSet()
        self.ts.add(Taxon("Amborella trichopoda", "Baill."))
        
        self.taxa = [Taxon("Amborella trichopoda", "Norbert"),
                     Taxon("Amborella trichopode", "Baill."),
                     Taxon("Amborella trichopoda subsp. trichopoda", "Baill."),
                     Taxon("Amborella figmentimagionatium", "Me"),
                    ]
    
    def do_match(self, tracker):
        match_taxa(self.taxa, self.ts, tracker=tracker)
    
    def test_csv_tracker(self):
        sio = StringIO()
        track = tracker.CSVTracker(sio)
        self.do_match(track)
        
        sio.seek(0)
        #print(sio.getvalue()) #DBG
        reader = csv.reader(sio)
        next(reader)
        events = [r[5] for r in reader]
        self.assertEqual(events, ['authority overlooked', 'one match',  # wrong authority
        'authority overlooked', 'fuzzy match', 'name variation', 'one match', # Mis-spelling
        'authority overlooked', 'upgraded subspecies', 'name variation', 'one match', # subspecies
        'authority overlooked', 'no match',   # non-matching taxon
        ])
    
    def test_csv_matches(self):
        sio = StringIO()
        track = tracker.CSVListMatches(sio)
        self.do_match(track)
        
        sio.seek(0)
        print(sio.getvalue()) #DBG
        reader = csv.reader(sio)
        next(reader)
        matched_names = [(r[2],r[3]) for r in reader]
        self.assertEqual(matched_names[:3], [("Amborella trichopoda", "Baill.")]*3)
        self.assertEqual(matched_names[3], ("",""))



from pyparsing import (Word, alphas, alphas8bit, Optional, Regex, Combine, 
               MatchFirst, ZeroOrMore, delimitedList, Literal, FollowedBy,
               Group, ParseResults, ParseException)
latinchars = alphas + alphas8bit
upper = "".join(c for c in latinchars if c.isupper())
lower = "".join(c for c in latinchars if c.islower())

# Species name components
genus = Regex("[A-Z][\w\-]*").setResultsName("genus")
sp_epithet = Word(latinchars + "-").setResultsName("sp")
species = genus + sp_epithet
subtax_rank = MatchFirst([Literal(s) for s in ("ssp", "subsp", "var", "f", "forma")]).setResultsName("subtax_rank")
subtax_rank_abbr = Combine(subtax_rank + Optional("."))
subtax_epithet = Word(latinchars + "-").setResultsName("subtax")
subtax = Group(subtax_rank_abbr + subtax_epithet)

# Author components
authsep = MatchFirst([Literal(s) for s in (",","&","and")]).suppress()
initial_letter = Word(upper, max=1).setResultsName("initial")
initials = ZeroOrMore(Combine(initial_letter + ".") + FollowedBy(Word(upper, latinchars)))
surname = Combine(Word(upper, latinchars) + Optional(".")).setResultsName("surname")
filius = Optional("f." + FollowedBy(authsep | subtax_rank)).setResultsName("filius")
author = Group(initials + surname + filius)

authorgroup = author + ZeroOrMore(authsep + author)

# Authority components
pp = Optional(Optional(",") + "p.p.")
nonauth = Optional("non" + authorgroup.setResultsName("nonauth"))
basauth = Optional("("+authorgroup.setResultsName("basauth")+")")
exauth = Optional("ex." + authorgroup.setResultsName("exauth"))
year = Optional(Optional(",") + Regex(r"\d{4}").setResultsName("year"))
mainauth = authorgroup.setResultsName("mainauth")
authority = Group(basauth + mainauth + exauth + nonauth).setResultsName("authority")

name_with_authority = species + Optional(Optional(authority) + subtax) + authority
name_without_authority = species + ZeroOrMore(subtax)

def parse_name(s, with_authority=True):
    if with_authority:
        try:
            res = name_with_authority.parseString(s)
        except ParseException:
            raise ValueError(s)
        yield res.genus, None
        if res[2].getName() == 'authority':
            yield res.sp, res[2]
            res = res[3:]
        else:
            yield res.sp, None
            res = res[2:]
        while res:
            subtax, auth, *res = res
            yield subtax, auth
    
    else: # No authority
        res = name_without_authority.parseString(s)
        for part in res:
            yield part, None

# Stashed here - this was attached to taxonome.taxa.base.Name

@classmethod
def from_string_new(cls, s):
    (genus,_), (sp, auth), *subpairs = parse_name(s)
    auths = [auth]
    subnames = []
    for namepart, auth in subpairs:
        #if namepart == 'ssp.': namepart = 'subsp.'
        subnames.append(namepart)
        auths.append(auth)
    name = cls([genus, sp] + subnames,"")# auths[-1])
    if subnames and len(auths) > 1 and auths[-2]:
        name.parent.authority = auths[-2]
    return name
    



from .qt import QtCore, QtGui

class DatasetItem(QtGui.QStandardItem):
    search_term = None
    search_results = None
    
    def __init__(self, name, ds, filename=None, **kwargs):
        super().__init__()
        self.ds = ds
        self.filename = filename
        self.name = name
        self.setData(name, QtCore.Qt.EditRole)
    
    def data(self, role=QtCore.Qt.UserRole+1):
        if role == QtCore.Qt.DisplayRole:
            return "{} ({} taxa)".format(self.name, len(self.ds))
        return super().data(role)
    
    def update_name(self):
        self.name = self.data(QtCore.Qt.EditRole)
    
    @property
    def autofilename(self):
        return self.filename or self.name.replace(" ","_")
    
    def copy(self):
        return type(self)(self.name, self.ds, self.filename)

class TaxonItem(QtGui.QStandardItem):
    def __init__(self, text, taxon):
        super().__init__(text)
        self.taxon = taxon

class TaxaItemModel(QtGui.QStandardItemModel):
    def __init__(self, *args):
        super().__init__(*args)
        self.setColumnCount(2)
        self.setHeaderData(0, QtCore.Qt.Horizontal, "Name")
        self.setHeaderData(1, QtCore.Qt.Horizontal, "Authority")

class NamePairItem(QtGui.QStandardItem):
    def __init__(self, namepair):
        self.namepair = namepair
        name, accname = namepair
        if name == accname:
            text = str(name)
        else:
            text = "{an}\n   from {n}".format(n=name, an=accname)
        super().__init__(text)



import warnings
from .collection import TaxonSet
from .base import Taxon

def add_info_nested(taxon, info, ds_name):
    """Passed to combine_datasets as the add_info parameter, to nest information
    by source dataset.
    """
    taxon.info[ds_name] = info

def add_info_flat(taxon, info, ds_name):
    """Passed to combine_datasets as the add_info parameter, to combine
    information into a flat dictionary.
    """
    for k, v in info.items():
        if k in taxon.info:
            k = str(k) + ' ['+ds_name+']'
        taxon.info[k] = v

def combine_datasets(target_ds, background_ds, distrib_ds_name,
                                                    add_info=add_info_nested):
    """Combine several taxa datasets into one.
    
    Parameters:
        target_ds, background_ds :
          Each should be a sequence of 2-tuples, (name, taxonset). The name will be
          used as the key in the taxa's info. All taxa from any target dataset are
          included in the output, whereas information from background datasets is
          only copied to taxa in target datasets. There must be at least one target
          dataset, and at least one other (either target or background).
          
          Target datasets may be any iterable collection of taxa. Background datasets
          should be TaxonSets or similar.
        
        distrib_ds_name : str
          The name of the dataset from which distribution information should be
          taken, or 'all' to combine distribution info from all datasets.
        
        add_info : func
          A function to combine the information dicts for a taxon. It takes 
          three parameters: the taxon object, the information dict to be added,
          and the name of a dataset.
          
          The default will nest information under the names of the datasets. The
          :func:`add_info_flat` function will make a flat dictionary.
    
    Returns: A :class:`TaxonSet`.
    """
    def _check_distrib_ds(dsname):
        return (dsname == distrib_ds_name) or (distrib_ds_name == 'all')

    output = TaxonSet()

    for dsname, ds in target_ds:
        is_distrib_ds = _check_distrib_ds(dsname)
        for tax in ds:
            if tax.name in output:
                t2 = output[tax.name]
                add_info(t2, tax.info, dsname)
                if is_distrib_ds:
                    t2.distribution.update(tax.distribution)
            else:
                newtax = Taxon(tax.name)
                add_info(newtax, tax.info, dsname)
                if is_distrib_ds:
                    newtax.distribution.update(tax.distribution)
                output.add(newtax)

    for dsname, ds in background_ds:
        is_distrib_ds = _check_distrib_ds(dsname)
        for tax in output:
            try:
                t2 = ds.get_by_accepted_name(tax.name)
            except KeyError:
                warnings.warn("Some taxon could not be found in a background "
                              "dataset - you may need to match the taxa first.")
                continue
            
            add_info(tax, t2.info, dsname)
            if is_distrib_ds:
                tax.distribution.update(t2.distribution)
    
    return output
